package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.marking.assembler.PaperAssembler;
import cn.com.qmth.stmms.ms.marking.dto.PaperDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * @author by xf
 * @date 2019/11/5.
 */
@RestController
@RequestMapping("/api/task")
public class TaskApi {

    @Autowired
    private PaperAssembler paperAssembler;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private MarkTaskRepo markTaskRepo;

    /**
     * 任务管理查询
     * @param workId
     * @param secretNumber
     * @param type
     * @return
     */
    @RequestMapping("/search")
    public PaperDTO searchBySecretNumber(@RequestParam("workId") Long workId,
                                         @RequestParam("secretNumber") String secretNumber,
                                         @RequestParam("type") String type){
        PaperDTO paperDTO = null;
        if("paper".equals(type)){
            Specification<Paper> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"),workId));
                if(secretNumber != null){
                    predicates.add(builder.equal(root.get("secretNumber"), secretNumber));
                }

                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            Optional<Paper> paperOptional = paperRepo.findAll(specification,new PageRequest(0,1))
                    .getContent().stream().findFirst();
            if(paperOptional.isPresent()){
                paperDTO = paperAssembler.toDTO(paperOptional.get());
            }
        } else if ("task".equals(type)){
            Specification<MarkTask> specification = (root,query,builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                if(secretNumber != null){
                    predicates.add(builder.equal(root.get("secretNumber"),secretNumber));
                }
                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            Optional<MarkTask> taskOptional = markTaskRepo.findAll(specification,new PageRequest(0,1))
                    .getContent().stream().findFirst();
            if(taskOptional.isPresent()){
                paperDTO = paperAssembler.toDTO(taskOptional.get().getPaper());
            }
        }
        return paperDTO;

    }

    /**
     * 重评
     * @param paperId
     * @param ranges
     * @return
     */
    @RequestMapping("/reevaluation")
    public boolean reevaluation(@RequestParam("paperId") Long paperId,
                                @RequestParam("rangs") String ranges){
        Paper paper = paperRepo.findOne(paperId);
//        paper.reject(null);
        //重评后撤销标准卷
//        paper.setSample(false);
        List<MarkTask> markTasks = markTaskRepo.findByPaperIdAndStage(paper.getId(), MarkStage.LEVEL);
        for (MarkTask markTask : markTasks) {
            String[] range = ranges.split(",");
            if(range.length > 0){
                for (String r : range) {
                    if(Objects.equals(String.valueOf(markTask.getMarkerId()), r)){
//                        markTask.setRejected(true);
//                        markTask.setOriginLevel(markTask.getResult());
                        markTask.setResult(null);
                        markTask.setActive(false);
                    }
                }
            }
        }
        markTaskRepo.save(markTasks);
//        paperRepo.save(paper);
        return true;
    }

}
